bl_info = {
    "name": "Uber Compositor",
    "description": "Uber Compositor",
    "author": "Abdou Bouam",
    "version": (1, 3, 4),
    "blender": (3,0,0),
    "location": "Node Editors > Add > Uber Compositor",
    "description": "Uber Compositor",
    "warning": "",
    "doc_url": "",
    "category": "Node",
}
from .download import download_file
from pathlib import Path
down_path = Path(__file__).parent
download_file("addons_file/uber_compositor/uber_compositor_nodes.blend",down_path)

import os
import bpy
from bpy.types import (
    Operator,
    Menu,
)
def printd(message): #only for debug
    print(message)
    pass
from bpy.app.handlers import persistent
    
blend_path = "/uber_compositor_nodes.blend"
suffix = " - UC"
node_names = ["Color Adjustments","Effects","Soft Glow","Glare",] # order matters!
extra_nodes = [
    "Anamorphic Lens Flare",
    "Black and White",
    "Bleach Bypass",
    "Color Temperature",
    "Dark Blue",
    "Dreamy",
    "Film Grain",
    "Grunge Overlay",
    "Highlight Fringe",
    "Lens Dirt",
    "Orange Teal",
    "Sepia",
    "Side Highlights",
    "Vignette",
    "Circular Blur",
    ] # Extra nodes 
resolution_dependant_nodes=["Effects - UC", "Vignette - UC", "Dreamy - UC","Circular Blur - UC"]

@persistent 
def update_resolution(scene):
    nodetree = bpy.context.scene.node_tree
    printd("changing resolution")
    for node in nodetree.nodes:
        if node.type == "GROUP" and node.node_tree.name in resolution_dependant_nodes:
            percentage = bpy.context.scene.render.resolution_percentage
            resolution_x = bpy.context.scene.render.resolution_x * percentage / 100
            resolution_y = bpy.context.scene.render.resolution_y * percentage / 100

            node.inputs["Image Width"].default_value = resolution_x
            node.inputs["Image Height"].default_value = resolution_y
            
            
def generate_enum(self,context):
    enum_list = []
    enum_list.append(("UberCompositorStandard", "Uber Compositor (Standard Setup)", "Add Uber Compositor (Standard Setup)"))
    for node_name in node_names + extra_nodes:
        enum_list.append((node_name.replace(" "," "),node_name, "add "+node_name))
    return enum_list
    
    
class NODE_OT_UberCompositor(bpy.types.Operator):
    bl_idname = "node.ubercompositor"
    bl_label = "ubercompositor"
    bl_description = "Add Uber Compositor"
    bl_options = {'REGISTER', 'UNDO','INTERNAL'}

    script_path = os.path.realpath(__file__)
    script_path = os.path.dirname(script_path)
    data_path = bpy.path.abspath(script_path + blend_path)
    
    def append_node_group(self,context,node_groups):
        uber_data_path = NODE_OT_UberCompositor.data_path+"/NodeTree"
        for node in node_groups:
            node_name = node + suffix
            appended_nodes = []
            for node_group in bpy.data.node_groups:
                if node_group.name[-5:]==suffix:
                    appended_nodes.append(node_group.name)
            if node_name in appended_nodes:
                printd("Node already exists :"+ node)
            else:
                bpy.ops.wm.append(directory=uber_data_path, link = False, filename=node_name)
                printd("appended "+ node)

    
    action: bpy.props.EnumProperty(items=generate_enum)
    

    def execute(self, context):
        node_groups=[]
        node_groups.append(self.action)
        if node_groups[0]=="UberCompositorStandard":
            node_groups = node_names
        printd("****************UBER COMPOSITOR*********************")
        NODE_OT_UberCompositor.append_node_group(self,context,node_groups)

        node_tree= context.space_data.node_tree
        try:
            nodes = node_tree.nodes
        except:
            self.report({"INFO"}, "请勾选使用节点！")
            return {'CANCELLED'}
        selected_nodes = context.selected_nodes
        links = node_tree.links
        
        offset = 300
        width = 200
        location = [0.0,0.0]

        nodetree = bpy.context.scene.node_tree
        composite = None
        viewer = None
        render_layers = None
        for node in nodetree.nodes:
            if not viewer and node.type == 'VIEWER':
                viewer = node
            if not composite and node.type == 'COMPOSITE':
                composite = node
            if not render_layers and  node.type == 'R_LAYERS':
                render_layers = node
        
        if render_layers and len(node_groups)>1:
            location 
            location[0] = render_layers.location[0]+offset
            location[1] = render_layers.location[1]
        

        elif len(selected_nodes):
            for node in selected_nodes:
                location[0] = location[0] + node.location[0]
                location[1] = location[1] + node.location[1]
            location[0] = location[0] / len(selected_nodes) + offset #?
            location[1] = location[1] / len(selected_nodes)
        bpy.ops.node.select_all(action='DESELECT')

        uber_nodes = [] # Keep created nodes here
        for i , node_group in enumerate(node_groups):
            uber_nodes.append(nodes.new("CompositorNodeGroup"))
            uber_node = uber_nodes[-1]
            uber_node.node_tree=bpy.data.node_groups[node_group + suffix ]
            uber_node.width = width
            uber_node.location[0] = location[0] + offset * i
            uber_node.location[1] = location[1]
            
            if len(uber_nodes) > 1: 
                links.new(uber_nodes[-2].outputs["Output"] , uber_nodes[-1].inputs["Input"])
        if len(uber_nodes) > 1:    
            if render_layers:
                links.new(render_layers.outputs["Image"] , uber_nodes[0].inputs["Input"])
            if composite:
                links.new(uber_nodes[-1].outputs["Output"] , composite.inputs["Image"])
                composite.location[0] = uber_nodes[-1].location[0] + offset
            if viewer:
                links.new(uber_nodes[-1].outputs["Output"] , viewer.inputs["Image"])                
                viewer.location[0] = uber_nodes[-1].location[0] + offset
    
        update_resolution(bpy.context.scene)            

        return {'FINISHED'}

class NODE_MT_UberCompositorMenu(Menu):
    bl_label = "Uber Compositor"

    def draw(self, context):
        layout = self.layout
        layout.operator_enum(NODE_OT_UberCompositor.bl_idname, "action")
        """for extra_node_name in NODE_OT_UberCompositor.node_names: #switch to extra_nodes later
            #NODE_OT_UberCompositor.add_node = extra_node_name
            layout.operator(NODE_OT_UberCompositor.bl_idname , text=extra_node_name).add_node = extra_node_name
            #NODE_OT_UberCompositor.add_node = None"""
        return

def add_uber_compositor_menu(self, context):
    if context.space_data.tree_type == "CompositorNodeTree":
        self.layout.menu(
            NODE_MT_UberCompositorMenu.__name__,
            text="Uber Compositor",
            icon='NODE_COMPOSITING',
        )                



def register():
    bpy.utils.register_class(NODE_MT_UberCompositorMenu)
    bpy.utils.register_class(NODE_OT_UberCompositor)
    bpy.types.NODE_MT_add.append(add_uber_compositor_menu)
    bpy.app.handlers.render_init.append(update_resolution)

def unregister():
    bpy.utils.unregister_class(NODE_MT_UberCompositorMenu)
    bpy.utils.unregister_class(NODE_OT_UberCompositor)
    bpy.types.NODE_MT_add.remove(add_uber_compositor_menu)
    bpy.app.handlers.render_init.remove(update_resolution)

if __name__ == "__main__":
    register()
